#ifndef hpp_pinocchio_idl__robots_hpp__
#define hpp_pinocchio_idl__robots_hpp__

//
// Implemention of IDL interfaces in file /local/robotpkg/var/tmp/robotpkg/path/py-hpp-corbaserver/work/hpp-corbaserver-4.8.0/idl/hpp/pinocchio_idl/robots.idl
//

#include <hpp/pinocchio_idl/robots-idl.hh>
#include "hpp/corbaserver/servant-base.hh"

#include <hpp/pinocchio/device.hh>
#include <hpp/pinocchio/configuration.hh>
#include <hpp/pinocchio/liegroup.hh>
#include <hpp/pinocchio/center-of-mass-computation.hh>


namespace hpp {

namespace corbaServer {


//
// Class implementing IDL interface hpp::pinocchio_idl::CenterOfMassComputation
//
namespace pinocchio_impl {
template <typename _Base, typename _Storage>
class CenterOfMassComputationServant: public ServantBase<hpp::pinocchio::CenterOfMassComputation, _Storage>, public virtual _Base
{
public:
  typedef hpp::pinocchio::CenterOfMassComputation HppBase;

  SERVANT_BASE_TYPEDEFS(hpp::pinocchio_idl::CenterOfMassComputation, HppBase);

public:
  // standard constructor
  CenterOfMassComputationServant(Server* server, const _Storage& s);
  virtual ~CenterOfMassComputationServant();

  // methods corresponding to defined IDL attributes and operations
  
  void compute ();


  hpp::floatSeq* com ();


  hpp::value_type mass ();


  hpp::floatSeqSeq* jacobian ();


};

typedef CenterOfMassComputationServant<POA_hpp::pinocchio_idl::CenterOfMassComputation,hpp::pinocchio::CenterOfMassComputationPtr_t > CenterOfMassComputation;
} // namespace pinocchio_impl


//
// Class implementing IDL interface hpp::pinocchio_idl::Device
//
namespace pinocchio_impl {
template <typename _Base, typename _Storage>
class DeviceServant: public ServantBase<hpp::pinocchio::Device, _Storage>, public virtual _Base
{
public:
  typedef hpp::pinocchio::Device HppBase;

  SERVANT_BASE_TYPEDEFS(hpp::pinocchio_idl::Device, HppBase);

public:
  // standard constructor
  DeviceServant(Server* server, const _Storage& s);
  virtual ~DeviceServant();

  // methods corresponding to defined IDL attributes and operations
  
  char* name ();


  void integrate (const hpp::floatSeq& config_in, const hpp::floatSeq& velocity, hpp::floatSeq_out result);


  void difference (const hpp::floatSeq& q1, const hpp::floatSeq& q2, hpp::floatSeq_out result);


  void interpolate (const hpp::floatSeq& q1, const hpp::floatSeq& q2, ::CORBA::Float u, hpp::floatSeq_out result);


  void saturate (const hpp::floatSeq& qin, hpp::floatSeq_out qout, hpp::boolSeq_out saturation);


};

typedef DeviceServant<POA_hpp::pinocchio_idl::Device,hpp::pinocchio::DevicePtr_t > Device;
} // namespace pinocchio_impl



} // namespace hpp

} // namespace corbaServer

#endif // hpp_pinocchio_idl__robots_hpp__

